function [AB,n1,n2,perm,fail] = specbip( A, n1, reorder, tau, zout, show)
% SPECBIP spectral bipartition of a graph
%
% [AB,n1,n2,perm,fail] = specbip( A, n1, reorder, tau, zout, show)
% input parameters:
%	A	adjacency matrix
%	n1	number of nodes in the largest set (optional)
%	reorder	reordering method (optional)
%	tau	threshold for separating the sets (optional)
%	zout	method to zero out elements (optional)
%	show	figure for displaying eigenvalues (optional)
% output parameters:
%	AB	approximated adjacency matrix
%	n1	cardinality of the largest set (estimated, if not provided)
%	n2	cardinality of the smallest set (estimated, if not provided)
%	perm	nodes permutation determined by the algorithm
%	fail	reason for failure of the method

weighted = any((A(:)~=1)&(A(:)~=0));

if (nargin<6) || isempty(show), show = 0; end
if (nargin<5) || isempty(zout), if weighted, zout = 2, else, zout = 1; end, end
if (nargin<4) || isempty(tau), tau = 0; end
if (nargin<3) || isempty(reorder), reorder = 1; end
if (nargin<2) || isempty(n1), haven1 = 0; else, haven1 = 1; end

if weighted		% weighted graph
	if zout~=2, warning('The value of zout may not be appropriate.'), end
else			% unweighted graph
	if zout~=1, warning('The value of zout may not be appropriate.'), end
end

n = size(A,1);
fail = 0;

% compute eigenvalues and sort them
[W0,D0] = eig(A);
[alpha,ppd] = sort(diag(D0),'descend');
%D0 = D0(ppd,ppd);	% useless
W0 = W0(:,ppd);
W0 = W0*diag(sign(W0(1,:)));

% estimate cardinality of the two sets (if unknown)
if tau
	n12 = sum(abs(alpha)<tau);
else
	alam = sort(abs(alpha));
	rho = alam(2:n)./alam(1:n-1);
	% jump>1e2 & |alam_successivo|>1e-8
	ijump = find((rho>1e2) & (alam(2:n)>1e-8));
	if isempty(ijump)
		warning('Cannot separate the two sets.')
		n12 = 0;
		fail = 1;
	else
		[mm im] = max(rho(ijump));
		n12 = ijump(im);
		if alam(n12) > 1e-3
			warning('Sets separation may be wrong.')
			fail = 2;
		end
	end
end
en2 = round((n-n12)/2);
en1 = n-en2;
if haven1
	n2 = n-n1;
else
	n1 = en1;
	n2 = en2;
end

if show
	figure(show)
	semilogy(sort(abs(alpha)),'o-')
	title('eigenvalues of the test matrix')
	grid
end

if n1<n2, nt = n2; n2 = n1; n1 = nt; end

if (reorder==1) && (n1==n2)
	warning('reorder=1 unaccettable, switching to reorder=2.')
	reorder = 2;
end

switch reorder

case 1
	%w = W0(:,[n2+1:n1]).^2 * ones(n1-n2,1);
	w = abs(W0(:,[n2+1:n1])) * ones(n1-n2,1);
	[wt pp] = sort(abs(w),'descend');
	W = W0(pp,:);

case 2
	%w = (W0(:,[1:n2])-W0(:,n:-1:n1+1)).^2 * ones(n2,1);
	w = abs(W0(:,[1:n2])-W0(:,n:-1:n1+1)) * ones(n2,1);
	[wt pp] = sort(abs(w));
	W = W0(pp,:);

case 3
	%w = (W0(:,[1:n2])+W0(:,n:-1:n1+1)).^2 * ones(n2,1);
	w = abs(W0(:,[1:n2])+W0(:,n:-1:n1+1)) * ones(n2,1);
	[wt pp] = sort(abs(w),'descend');
	W = W0(pp,:);

otherwise
	error('Invalid reordering method.')

end
perm = pp;

% approximate eigenvalues
beta = zeros(n,1);
for j = 1:n2
	beta(j) = (alpha(j)-alpha(n-j+1))/2;
	beta(n-j+1) = -beta(j);
end

W_11 = W(1:n1,1:n2);
W_12 = W(1:n1,n2+1:n1);
W_13 = W(1:n1,n1+1:n);
W_21 = W(n1+1:n,1:n2);
%W_22 = W(n1+1:n,n2+1:n1);
W_23 = W(n1+1:n,n1+1:n);

% eigenvectors approximation
XX = W_11+W_13(:,end:-1:1);
[Ux1,~,Vx1] = svd(XX,0);
X1 = Ux1*Vx1';
U1 = X1/sqrt(2);

[Ux2,~,Vx2] = svd(W_12,0);
X2 = Ux2*Vx2';
U2 = X2;

YY = W_21-W_23(:,end:-1:1);
[Uy,~,Vy] = svd(YY,0);
Y = Uy*Vy';
V = Y/sqrt(2);

Q = [U1 U2 U1(:,end:-1:1);V zeros(n2,n1-n2) -V(:,end:-1:1)];
Lam = diag(beta);
AB = Q*Lam*Q';

switch zout
case 0
	% don't zero out elements
case 1
	AB = double(AB>.5);	% quanto deve valere la soglia?
case 2
	AB(abs(AB)<1e-2) = 0; 
otherwise
	error('Unknown zero-out method.')
end

